use crate::util::bit;

#[derive(Clone, Copy)]
pub struct MoveSet {
    direction:u32,
    distance:u32,
}

#[derive(Clone, Copy)]
pub struct PieceClass {
    name:&'static str,
    moves:MoveSet,
    pmoves:MoveSet,
}

pub const PIECES_COUNT  :usize = 7;
pub const PIECE_NONE    :u8 = PIECES_COUNT as u8 + 1;
pub const PIECE_MILITIA :u8 = 0;
pub const PIECE_KNIGHT  :u8 = 1;
pub const PIECE_LANCE   :u8 = 2;
pub const PIECE_TOWER   :u8 = 3;
pub const PIECE_CASTLE  :u8 = 4;
pub const PIECE_DRAGON  :u8 = 5;
pub const PIECE_KING    :u8 = 6;

pub const PIECES :[PieceClass; PIECES_COUNT] = [
    PieceClass {
        name: "Militia",
        moves: MoveSet {
            direction:bit(0) | bit(1) | bit(5),
            distance:0,
        },
        pmoves: MoveSet{
            direction:bit(0) | bit(1) | bit(2) | bit(4) | bit(5),
            distance:0,
        },
    },
    PieceClass {
        name: "Knight",
        moves: MoveSet {
            direction:bit(3) | bit(6) | bit(11) | bit(13) | bit(17),
            distance:0,
        },
        pmoves: MoveSet{
            direction:bit(3) | bit(6) | bit(7) | bit(10) | bit(11) | bit(13) | bit(14) | bit(16) | bit(17),
            distance:0,
        },
    },
    PieceClass {
        name: "Lance",
        moves: MoveSet {
            direction:0,
            distance:0,
        },
        pmoves: MoveSet {
            direction:0,
            distance:0,
        },
    },
    PieceClass {
        name: "Tower",
        moves: MoveSet {
            direction:0,
            distance:0,
        },
        pmoves: MoveSet {
            direction:0,
            distance:0,
        },
    },
    PieceClass {
        name: "Castle",
        moves: MoveSet {
            direction:0,
            distance:0,
        },
        pmoves: MoveSet {
            direction:0,
            distance:0,
        },
    },
    PieceClass {
        name: "Dragon",
        moves: MoveSet {
            direction:0,
            distance:0,
        },
        pmoves: MoveSet {
            direction:0,
            distance:0,
        },
    },
    PieceClass {
        name: "King",
        moves: MoveSet {
            direction:0,
            distance:0,
        },
        pmoves: MoveSet {
            direction:0,
            distance:0,
        },
    },
];

#[derive(Clone, Copy)]
pub struct Piece {
    class:u8,
    promoted:bool,
}
impl Piece {
    pub fn new() -> Self
    {
        Self {
            class:PIECE_NONE,
            promoted:false,
        }
    }
}
