use crate::util::{pack_u16, unpack_u16};

use super::Packet;

#[derive(Clone)]
pub struct PacketAuth {
    pub handle:String,
    pub secret:String,
}
impl PacketAuth {
    pub fn new() -> Self
    {
        Self {
            handle:String::new(),
            secret:String::new(),
        }
    }
}
impl Packet for PacketAuth {
    type Data = Self;

    fn encode(&self) -> Vec<u8>
    {
        [
            pack_u16(self.handle.len() as u16),
            self.handle.as_bytes().to_vec(),
            pack_u16(self.secret.len() as u16),
            self.secret.as_bytes().to_vec(),
        ].concat()
    }

    fn decode(data:&Vec<u8>, index:&mut usize) -> Result<Self::Data, ()>
    {
        let mut result = Self::new();

        let mut length = unpack_u16(data, index) as usize;
        if data.len() >= *index + length {
            match String::from_utf8(data[*index..*index+length].to_vec()) {
                Ok(text) => {
                    *index += length;
                    result.handle = text;
                    
                    length = unpack_u16(data, index) as usize;
                    if data.len() >= *index + length {

                        match String::from_utf8(data[*index..*index+length].to_vec()) {
                            Ok(text) => {
                                *index += length;
                                result.secret = text;

                                return Ok(result);
                            }
                            Err(_) => { }
                        }
                    }
                }
                Err(_) => { }
            }
        }

        Err(())
    }
}


#[derive(Clone)]
pub struct PacketAuthResponse {
    pub status:bool,
    pub token:[u8; 8],
    pub secret:[u8; 16],
}
impl PacketAuthResponse {
    pub fn new() -> Self
    {
        Self {
            status:false,
            token:[0; 8],
            secret:[0; 16],
        }
    }
}
impl Packet for PacketAuthResponse {
    type Data = Self;

    fn encode(&self) -> Vec<u8>
    {
        vec![]
    }

    fn decode(_data:&Vec<u8>, _index:&mut usize) -> Result<Self::Data, ()>
    {
        Err(())
    }
}
