use game::util::pack_u32;
use pool::Pool;
use trie::Trie;

pub mod user; use user::User;
pub mod session; use session::Session;
pub mod contest; use contest::Contest;
pub mod context;

pub struct App {
    users:Pool<User>,
    handles:Trie<u32>,
    salts:Vec<[u8; 16]>,
    sessions:Pool<Session>,
    contests:Pool<Contest>,
}
impl App {
    pub fn new() -> Self
    {
        Self {
            users:Pool::new(),
            handles:Trie::new(),
            salts:Vec::new(),
            sessions:Pool::new(),
            contests:Pool::new(),
        }
    }

    pub fn init(&mut self) -> Result<(),std::io::Error>
    { use std::{path::Path, fs};
        
        let path_data = Path::new("data");
        if !path_data.exists() {
            fs::create_dir(path_data)?;
            fs::create_dir(path_data.join("c"))?;
            fs::create_dir(path_data.join("s"))?;
            fs::create_dir(path_data.join("u"))?;

            fs::write(path_data.join("c/.i"), pack_u32(0)).ok();
            fs::write(path_data.join("s/.i"), pack_u32(0)).ok();
            fs::write(path_data.join("u/.i"), pack_u32(0)).ok();
        }

        Ok(())
    }
}
