use std::net::SocketAddr;

use bus::Bus;

mod util;
mod app;
mod system;
mod protocol;

use app::App;
use hyper::{body::Bytes, upgrade::Upgraded};
use system::{cache::WebCache, net::Stream};
use tokio_stream::StreamExt;
use tokio_tungstenite::WebSocketStream;
use hyper_util::rt::TokioIo;
//use tokio_rustls::server::TlsStream;
//use tokio::net::TcpStream;

async fn thread_datasystem(mut _app:App, bus:Bus<protocol::QRPacket>)
{
    use protocol::QRPacket;
    use game::protocol::packet::*;

    while match bus.receive_wait() {
        Some(packet) => {
            match packet.data {
                QRPacket::QAuth(_request) => {
                    let response = PacketAuthResponse::new();

                    // get user id from handle

                    // get user salt

                    // hash request secret

                    // compare hash to user secret
                    
                    // return response
                    bus.send(packet.from, QRPacket::RAuth(response)).is_ok()
                }
                _ => { true }
            }
        }
        None => false,
    } { }
}

#[derive(Clone)]
struct HttpServiceArgs {
    bus:Bus<protocol::QRPacket>,
    cache:WebCache,
}

async fn handle_ws(mut ws_stream:WebSocketStream<TokioIo<Upgraded>>, args:HttpServiceArgs)
{
    use tokio_tungstenite::tungstenite::protocol::Message;
    use game::util::unpack_u16;
    use protocol::QRPacket;
    use game::protocol::{
        code::*,
        packet::*,
    };

    println!("ws ready");

    let bus_ds = args.bus.mailbox(1).unwrap_or(1);

    while match ws_stream.try_next().await {
        Ok(msg) => match msg {
            Some(Message::Binary(data)) => {
                
                let mut index :usize = 0;
                let code: u16 = unpack_u16(&data, &mut index);

                match code {
                    CODE_AUTH => {
                        match PacketAuth::decode(&data, &mut index) {
                            Ok(packet) => {
                                if args.bus.send(bus_ds, QRPacket::QAuth(packet)).is_ok() {
                                    match args.bus.receive_wait() {
                                        Some(resp) => match resp.data {
                                            QRPacket::RAuth(_resp) => {
                                                
                                            }
                                            _ => { }
                                        }
                                        None => { }
                                    }
                                }
                            }
                            Err(_) => { }
                        }
                    }
                    _ => { }
                }

                true
            }
            Some(_) => true,
            None => false
        }
        Err(_) => false,
    } { }

    ws_stream.close(None).await.ok();
}

async fn service_http(mut request:hyper::Request<hyper::body::Incoming>, args:HttpServiceArgs) -> Result<hyper::Response<http_body_util::Full<Bytes>>, std::convert::Infallible>
{
    use hyper::{Response, body::Bytes, header::{CONTENT_TYPE, CACHE_CONTROL}}; //SEC_WEBSOCKET_ACCEPT, SEC_WEBSOCKET_KEY, UPGRADE
    use http_body_util::Full;
    //use tokio_tungstenite::accept_async;
    //use tokio_tungstenite::tungstenite::handshake::derive_accept_key;

    println!("Serving: {}", request.uri().path());

    match request.uri().path() {
        "/.css" => Ok(Response::builder()
            .header(CONTENT_TYPE, "text/css")
            .header(CACHE_CONTROL, "no-cache")
            .body(Full::new(Bytes::from(args.cache.css()))).unwrap()),

        "/.js" => Ok(Response::builder()
            .header(CONTENT_TYPE, "text/javascript")
            .header(CACHE_CONTROL, "no-cache")
            .body(Full::new(Bytes::from(args.cache.js()))).unwrap()),

        "/favicon.png" => Ok(Response::builder()
            .header(CONTENT_TYPE, "image/png")
            .body(Full::new(Bytes::from(args.cache.favicon()))).unwrap()),

        _ => {
            if hyper_tungstenite::is_upgrade_request(&request) {
            //if request.headers().get(UPGRADE).map(|h| h == "websocket").unwrap_or(false) {
                //let response = create_response_with_body(&request, || Full::new(Bytes::new())).unwrap();

                //let key = request.headers().get(SEC_WEBSOCKET_KEY)
                //    .and_then(|v| v.to_str().ok())
                //    .map(|k| derive_accept_key(k.as_bytes()))
                //    .unwrap_or_default();

                if let Ok((response, websocket)) = hyper_tungstenite::upgrade(&mut request, None) {
                    tokio::task::spawn(async move {
                        match websocket.await {
                            Ok(websocket) => handle_ws(websocket, args).await,
                            Err(_) => { }
                        }
                        //match hyper::upgrade::on(request).await {
                            /*Ok(upgraded) => {
                                match upgraded.downcast::<TokioIo<TlsStream<TcpStream>>>() {
                                    Ok(parts) => {
                                        match accept_async(parts.io.into_inner()).await {
                                            Ok(ws_stream) => {
                                                println!("here");
                                                handle_ws(ws_stream, args).await
                                            }
                                            Err(e) => { println!("ws not accepted: {}", e.to_string()); }
                                        }
                                    }
                                    Err(_) => { println!("transfer error"); }
                                }
                            }
                            Er(e) => { println!("upgrade error: {}", e.to_string()); }*/
                        //}
                    });

                    Ok(response)
                } else {
                    Ok(Response::builder()
                        .status(401)
                        .body(Full::new(Bytes::new()))
                            .unwrap())
                }

                //Ok(Response::builder()
                //    .status(101)
                //    .header(CONNECTION, "Upgrade")
                //    .header(UPGRADE, "websocket")
                //    .header(SEC_WEBSOCKET_ACCEPT, key)
                //    .body(Full::new(Bytes::new()))
                //    .unwrap())
            } else {
                Ok(Response::builder()
                    .header(CONTENT_TYPE, "text/html")
                    .header(CACHE_CONTROL, "no-cache")
                    .body(Full::new(Bytes::from(args.cache.html()))).unwrap())
            }
        }
    }
}

async fn handle_http(stream:system::net::tls::TlsStream, addr:SocketAddr, args:HttpServiceArgs) -> Result<(),()>
{
    use hyper::server::conn::http1;
    use hyper::service::service_fn;

    println!("Connection from {}", addr.to_string());

    let io = TokioIo::new(stream.to_stream());

    let conn = http1::Builder::new()
        .serve_connection(io, service_fn(move |req| {
            service_http(req, args.clone())
        }));
    
    conn.with_upgrades().await.ok();
    Ok(())
}


#[tokio::main]
async fn main()
{
    use system::net::{
        Server,
        tls::*,
    };

    // Initialize application data.
    let mut app = App::new();
    if app.init().is_err() {
        println!("fatal: failed to initialize server.");
        return;
    }


    // Initialize central bus and data serivce.
    let b_main :Bus<protocol::QRPacket> = Bus::new_as(bus::Mode::Transmitter);
    match b_main.connect() {
        Ok(bus) => {
            tokio::spawn(async move {
                thread_datasystem(app, bus).await;
            });
        }
        Err(_) => {
            println!("fatal: failed to initialize bus.");
            return;
        }
    }


    // Initialize HTTPS service.
    match b_main.connect() {
        Ok(bus) => {
            let cache = WebCache::init();

            let mut server = TlsServer::new();
            if server.add_cert("omen.kirisame.com", "cert/fullchain.pem", "cert/privkey.pem").await.is_ok() {
                println!("Loaded cert file.");
            }
            match server.bind("0.0.0.0:38612").await {
                Ok(_) => {
                    println!("Listener bind successful.");
                    tokio::spawn(async move {
                        while server.accept(handle_http, HttpServiceArgs {
                            bus:bus.connect().unwrap(),
                            cache:cache.clone(),
                        }).await.is_ok() { }
                    });
                }
                Err(_) => {
                    println!("error: failed to bind port 38612.");
                }
            }
        }
        Err(_) => {
            println!("error: failed to initialize HTTPS service.");
        }
    }

    std::thread::park();
}
