use std::{io::Read, sync::{Arc, RwLock}};

use crate::util::string::minimize_whitespace;

struct WebCacheData {
    html:String,
    css:String,
    js:String,
    favicon:Vec<u8>,
}

#[derive(Clone)]
pub struct WebCache {
    data:Arc<RwLock<WebCacheData>>,
}
impl WebCache {
    pub fn init() -> Self
    {
        use std::fs::File;

        let mut html = String::new();
        let mut css = String::new();
        let mut js = String::new();
        let mut favicon = Vec::<u8>::new();

        // Cache html file
        match File::open("www/.html") {
            Ok(mut file) => {
                file.read_to_string(&mut html).ok();
                html = minimize_whitespace(&html);
            }
            Err(_) => { }
        }

        // Cache css file
        match File::open("www/.css") {
            Ok(mut file) => {
                file.read_to_string(&mut css).ok();
                css = minimize_whitespace(&css);
            }
            Err(_) => { }
        }

        // Cache js file
        match File::open("www/.js") {
            Ok(mut file) => {
                file.read_to_string(&mut js).ok();
                //js = minimize_whitespace(&js);
            }
            Err(_) => { }
        }

        // Cache favicon file
        match File::open("www/favicon.png") {
            Ok(mut file) => {
                file.read_to_end(&mut favicon).ok();
            }
            Err(_) => { }
        }

        Self {
            data:Arc::new(RwLock::new(WebCacheData {
                html, css, js, favicon,
            })),
        }
    }

    pub fn html(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.html.to_string(),
            Err(_) => String::new(),
        }
    }

    pub fn css(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.css.to_string(),
            Err(_) => String::new(),
        }
    }

    pub fn js(&self) -> String
    {
        match self.data.read() {
            Ok(reader) => reader.js.to_string(),
            Err(_) => String::new(),
        }
    }

    pub fn favicon(&self) -> Vec<u8>
    {
        match self.data.read() {
            Ok(reader) => reader.favicon.clone(),
            Err(_) => Vec::new(),
        }
    }
}
